require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_merchant_checkout/models/encrypted_payment_data_response'
 

module MastercardMerchantCheckout
 module Api

  class EncryptedPaymentDataApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMerchantCheckout::Tracker


    # EncryptedPaymentData Service
    # Use this service to retrieve the encrypted consumer’s payment card, and shipping details from Masterpass. API URL: /masterpass/encrypted-paymentdata/{transactionId}
    # @param transaction_id Identifies the transaction for which to return the consumer’s payment data. This is the oauth_verifier value sent by Masterpass in the callback URL after the Masterpass UI is closed.
    # @param query_params The query parameters.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [EncryptedPaymentDataResponse]
    def self.show(transaction_id, query_params, api_config = nil)
		 path = "/masterpass/encrypted-paymentdata/{transactionId}"
	 	 service_request = ServiceRequest.new
     service_request.path_params["transactionId"] = transaction_id
	 
	   
	 service_request.query_params = query_params
	   service_request.content_type = "application/json"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "GET",EncryptedPaymentDataResponse)
    end

  end

 end
end



