require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_merchant_checkout/models/express_checkout_request'
require_relative '../../mastercard_merchant_checkout/models/payment_data'
 

module MastercardMerchantCheckout
 module Api

  class ExpressCheckoutApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMerchantCheckout::Tracker


    # ExpressCheckout Service
    # Use this service to retrieve the consumer’s full payment card, shipping address, and recipient personal information from Masterpass when performing an Express Checkout. If the merchant is PCI-compliant, then this API will return the complete payload. If the merchant is not PCI-compliant, then this API will only return non-PCI data. In both cases, a new pairingId will be returned to the merchant. API URL: /masterpass/expresscheckout
    # @param express_checkout_request Express Checkout Request.

    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [PaymentData]
    def self.create(express_checkout_request, api_config = nil)
		 path = "/masterpass/expresscheckout"
	 	 service_request = ServiceRequest.new
     
	   
	   service_request.body = express_checkout_request
	 
	   service_request.content_type = "application/json"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "POST",PaymentData)
    end

  end

 end
end



