require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_merchant_checkout/models/payment_data'
 

module MastercardMerchantCheckout
 module Api

  class PaymentDataApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMerchantCheckout::Tracker


    # PaymentData Service
    # Use this service to retrieve the consumer’s payment card, and shipping details from Masterpass. New or existing Brazil merchants looking for the house number and street name as separate fields should leverage these two additional extension points in their Masterpass checkout integration. Line 4 will provide the house number and line 5 will provide the street name. Brazil merchants should first look for data in lines 4 and 5, and in the case that no data is found there, merchants should then look for data in line 1. API URL: /masterpass/paymentdata/{transactionId}
    # @param transaction_id Identifies the transaction for which to return the consumer’s payment data. This is the oauth_verifier value sent by Masterpass in the callback URL after the Masterpass UI is closed.
    # @param query_params The query parameters.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [PaymentData]
    def self.show(transaction_id, query_params, api_config = nil)
		 path = "/masterpass/paymentdata/{transactionId}"
	 	 service_request = ServiceRequest.new
     service_request.path_params["transactionId"] = transaction_id
	 
	   
	 service_request.query_params = query_params
	   service_request.content_type = "application/json"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "GET",PaymentData)
    end

  end

 end
end



