require 'date'
require 'roxml'
 

module MastercardMerchantCheckout
  # This class contains various methods to set express checkout request parameters required for ExpressCheckoutApi.
  class ExpressCheckoutRequest
    include ROXML
    
    xml_name "ExpressCheckoutRequest"
    
    # @!attribute checkout_id
    # @return [String] the merchant Checkout identifier.
    xml_accessor :checkout_id, :from =>"checkoutId"
   
    # @!attribute pairing_id
    # @return [String] the unique pairing token identifier used to fetch card and address data from a wallet that is paired with a merchant during Express Checkout.
    xml_accessor :pairing_id, :from =>"pairingId"
   
    # @!attribute pre_checkout_transaction_id
    # @return [String] the precheckout identifier from PreCheckoutApi response.
    xml_accessor :pre_checkout_transaction_id, :from =>"preCheckoutTransactionId"
   
    # @!attribute amount
    # @return [Float] the transaction amount as a decimal (100.50 for $100.50).
    xml_accessor :amount, :from =>"amount"
   
    # @!attribute currency
    # @return [String] the ISO-4217 code for currency of the transaction.
    xml_accessor :currency, :from =>"currency"
   
    # @!attribute card_id
    # @return [String] the card identifier from PreCheckoutApi response.
    xml_accessor :card_id, :from =>"cardId"
   
    # @!attribute shipping_address_id
    # @return [String] the shippingAddress identifier from PreCheckoutApi response.
    xml_accessor :shipping_address_id, :from =>"shippingAddressId"
   
    # @!attribute digital_goods
    # @return [BOOLEAN] the flag to indicate digital goods are being purchased so a shipping address is not required for the transaction.
    xml_accessor :digital_goods, :from =>"digitalGoods"
   
    # @!attribute psp_id
    # @return [String] the pspId passed by merchant to select PSP for pushing payment data.
    xml_accessor :psp_id, :from =>"pspId"
   
    # @!attribute external_data
    # @return [String] the external data to be used by PSP during Payment Processing. This parameter only used in PSP Push flow.
    xml_accessor :external_data, :from =>"externalData"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :checkout_id => :checkoutId ,
        :pairing_id => :pairingId ,
        :pre_checkout_transaction_id => :preCheckoutTransactionId ,
        :amount => :amount ,
        :currency => :currency ,
        :card_id => :cardId ,
        :shipping_address_id => :shippingAddressId ,
        :digital_goods => :digitalGoods ,
        :psp_id => :pspId ,
        :external_data => :externalData 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:checkout_id)
        self.checkout_id = attributes[:checkout_id]
      end
      
      if attributes.has_key?(:pairing_id)
        self.pairing_id = attributes[:pairing_id]
      end
      
      if attributes.has_key?(:pre_checkout_transaction_id)
        self.pre_checkout_transaction_id = attributes[:pre_checkout_transaction_id]
      end
      
      if attributes.has_key?(:amount)
        self.amount = attributes[:amount]
      end
      
      if attributes.has_key?(:currency)
        self.currency = attributes[:currency]
      end
      
      if attributes.has_key?(:card_id)
        self.card_id = attributes[:card_id]
      end
      
      if attributes.has_key?(:shipping_address_id)
        self.shipping_address_id = attributes[:shipping_address_id]
      end
      
      if attributes.has_key?(:digital_goods)
        self.digital_goods = attributes[:digital_goods]
      end
      
      if attributes.has_key?(:psp_id)
        self.psp_id = attributes[:psp_id]
      end
      
      if attributes.has_key?(:external_data)
        self.external_data = attributes[:external_data]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          checkout_id == o.checkout_id &&
          pairing_id == o.pairing_id &&
          pre_checkout_transaction_id == o.pre_checkout_transaction_id &&
          amount == o.amount &&
          currency == o.currency &&
          card_id == o.card_id &&
          shipping_address_id == o.shipping_address_id &&
          digital_goods == o.digital_goods &&
          psp_id == o.psp_id &&
          external_data == o.external_data
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [checkout_id, pairing_id, pre_checkout_transaction_id, amount, currency, card_id, shipping_address_id, digital_goods, psp_id, external_data].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantCheckout.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "ExpressCheckoutRequest" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :checkout_id => 'String',
        :pairing_id => 'String',
        :pre_checkout_transaction_id => 'String',
        :amount => 'Float',
        :currency => 'String',
        :card_id => 'String',
        :shipping_address_id => 'String',
        :digital_goods => 'BOOLEAN',
        :psp_id => 'String',
        :external_data => 'String'
        
      }
    end
  end


end
