require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_checkout/models/address'
require_relative '../../mastercard_merchant_checkout/models/authentication_options'
require_relative '../../mastercard_merchant_checkout/models/card'
require_relative '../../mastercard_merchant_checkout/models/personal_info'
require_relative '../../mastercard_merchant_checkout/models/tokenization'
 

module MastercardMerchantCheckout
  # This class contains various methods to get paymentData response parameters returned by PaymentDataApi.
  class PaymentData
    include ROXML
    
    xml_name "PaymentData"
    
    # @!attribute card
    # @return [Card] the payment card details.
    xml_accessor :card, :from =>"card",:as => Card
   
    # @!attribute shipping_address
    # @return [Address] the shipping address details.
    xml_accessor :shipping_address, :from =>"shippingAddress",:as => Address
   
    # @!attribute personal_info
    # @return [PersonalInfo] the recipient's personal information.
    xml_accessor :personal_info, :from =>"personalInfo",:as => PersonalInfo
   
    # @!attribute tokenization
    # @return [Tokenization] the tokenized card information.
    xml_accessor :tokenization, :from =>"tokenization",:as => Tokenization
   
    # @!attribute wallet_id
    # @return [String] the unique identifier for a wallet that helps identify the origin wallet for the transaction.
    xml_accessor :wallet_id, :from =>"walletId"
   
    # @!attribute wallet_name
    # @return [String] the wallet name.
    xml_accessor :wallet_name, :from =>"walletName"
   
    # @!attribute authentication_options
    # @return [AuthenticationOptions] the authentication information.
    xml_accessor :authentication_options, :from =>"authenticationOptions",:as => AuthenticationOptions
   
    # @!attribute pairing_id
    # @return [String] the new pairingId.
    xml_accessor :pairing_id, :from =>"pairingId"
   
    # @!attribute psp_transaction_id
    # @return [String] the new pspTransactionId.
    xml_accessor :psp_transaction_id, :from =>"pspTransactionId"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :card => :card ,
        :shipping_address => :shippingAddress ,
        :personal_info => :personalInfo ,
        :tokenization => :tokenization ,
        :wallet_id => :walletId ,
        :wallet_name => :walletName ,
        :authentication_options => :authenticationOptions ,
        :pairing_id => :pairingId ,
        :psp_transaction_id => :pspTransactionId 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:card)
        self.card = attributes[:card]
      end
      
      if attributes.has_key?(:shipping_address)
        self.shipping_address = attributes[:shipping_address]
      end
      
      if attributes.has_key?(:personal_info)
        self.personal_info = attributes[:personal_info]
      end
      
      if attributes.has_key?(:tokenization)
        self.tokenization = attributes[:tokenization]
      end
      
      if attributes.has_key?(:wallet_id)
        self.wallet_id = attributes[:wallet_id]
      end
      
      if attributes.has_key?(:wallet_name)
        self.wallet_name = attributes[:wallet_name]
      end
      
      if attributes.has_key?(:authentication_options)
        self.authentication_options = attributes[:authentication_options]
      end
      
      if attributes.has_key?(:pairing_id)
        self.pairing_id = attributes[:pairing_id]
      end
      
      if attributes.has_key?(:psp_transaction_id)
        self.psp_transaction_id = attributes[:psp_transaction_id]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          card == o.card &&
          shipping_address == o.shipping_address &&
          personal_info == o.personal_info &&
          tokenization == o.tokenization &&
          wallet_id == o.wallet_id &&
          wallet_name == o.wallet_name &&
          authentication_options == o.authentication_options &&
          pairing_id == o.pairing_id &&
          psp_transaction_id == o.psp_transaction_id
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [card, shipping_address, personal_info, tokenization, wallet_id, wallet_name, authentication_options, pairing_id, psp_transaction_id].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantCheckout.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "PaymentData" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :card => 'Card',
        :shipping_address => 'Address',
        :personal_info => 'PersonalInfo',
        :tokenization => 'Tokenization',
        :wallet_id => 'String',
        :wallet_name => 'String',
        :authentication_options => 'AuthenticationOptions',
        :pairing_id => 'String',
        :psp_transaction_id => 'String'
        
      }
    end
  end


end
