require 'date'
require 'roxml'
 

module MastercardMerchantCheckout
  # This class contains different methods to send transaction details to Masterpass using PostbackApi.
  class Postback
    include ROXML
    
    xml_name "Postback"
    
    # @!attribute transaction_id
    # @return [String] the transaction identifiers. Identifies the transaction for which to return the consumer’s payment data. This is the oauth_verifier value sent by Masterpass in callback URL after the Masterpass user interface is closed.
    xml_accessor :transaction_id, :from =>"transactionId"
   
    # @!attribute currency
    # @return [String] the ISO-4217 code for currency of the transaction.
    xml_accessor :currency, :from =>"currency"
   
    # @!attribute amount
    # @return [Float] the transaction amount as a decimal (100.50 for $100.50).
    xml_accessor :amount, :from =>"amount"
   
    # @!attribute payment_successful
    # @return [BOOLEAN] the payment status indicator. It is set to true if payment successful with payment processor else false.
    xml_accessor :payment_successful, :from =>"paymentSuccessful"
   
    # @!attribute payment_code
    # @return [String] the six-digit approval code returned by payment API.
    xml_accessor :payment_code, :from =>"paymentCode"
   
    # @!attribute payment_date
    # @return [DateTime] the date of purchase.
    xml_accessor :payment_date, :from =>"paymentDate"
   
    # @!attribute pre_checkout_transaction_id
    # @return [String] the preCheckoutTransactionId from the ExpressCheckout response.
    xml_accessor :pre_checkout_transaction_id, :from =>"preCheckoutTransactionId"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :transaction_id => :transactionId ,
        :currency => :currency ,
        :amount => :amount ,
        :payment_successful => :paymentSuccessful ,
        :payment_code => :paymentCode ,
        :payment_date => :paymentDate ,
        :pre_checkout_transaction_id => :preCheckoutTransactionId 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:transaction_id)
        self.transaction_id = attributes[:transaction_id]
      end
      
      if attributes.has_key?(:currency)
        self.currency = attributes[:currency]
      end
      
      if attributes.has_key?(:amount)
        self.amount = attributes[:amount]
      end
      
      if attributes.has_key?(:payment_successful)
        self.payment_successful = attributes[:payment_successful]
      end
      
      if attributes.has_key?(:payment_code)
        self.payment_code = attributes[:payment_code]
      end
      
      if attributes.has_key?(:payment_date)
        self.payment_date = attributes[:payment_date]
      end
      
      if attributes.has_key?(:pre_checkout_transaction_id)
        self.pre_checkout_transaction_id = attributes[:pre_checkout_transaction_id]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          transaction_id == o.transaction_id &&
          currency == o.currency &&
          amount == o.amount &&
          payment_successful == o.payment_successful &&
          payment_code == o.payment_code &&
          payment_date == o.payment_date &&
          pre_checkout_transaction_id == o.pre_checkout_transaction_id
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [transaction_id, currency, amount, payment_successful, payment_code, payment_date, pre_checkout_transaction_id].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantCheckout.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Postback" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :transaction_id => 'String',
        :currency => 'String',
        :amount => 'Float',
        :payment_successful => 'BOOLEAN',
        :payment_code => 'String',
        :payment_date => 'DateTime',
        :pre_checkout_transaction_id => 'String'
        
      }
    end
  end


end
