require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_checkout/models/contact_info'
require_relative '../../mastercard_merchant_checkout/models/pre_checkout_card'
require_relative '../../mastercard_merchant_checkout/models/shipping_address'
 

module MastercardMerchantCheckout
  # This class contains different methods for different types of precheckout data like card, contact and shipping address details.
  class PreCheckoutData
    include ROXML
    
    xml_name "PreCheckoutData"
    
    # @!attribute cards
    # @return [Array<PreCheckoutCard>] the cards in the paired wallet.
    xml_accessor :cards, :from =>"cards", :as =>[PreCheckoutCard]
   
    # @!attribute shipping_addresses
    # @return [Array<ShippingAddress>] the shipping addresses details.
    xml_accessor :shipping_addresses, :from =>"shippingAddresses", :as =>[ShippingAddress]
   
    # @!attribute contact_info
    # @return [ContactInfo] the contact info details.
    xml_accessor :contact_info, :from =>"contactInfo",:as => ContactInfo
   
    # @!attribute pre_checkout_transaction_id
    # @return [String] the preCheckout transaction id.
    xml_accessor :pre_checkout_transaction_id, :from =>"preCheckoutTransactionId"
   
    # @!attribute consumer_wallet_id
    # @return [String] the consumer’s wallet id.
    xml_accessor :consumer_wallet_id, :from =>"consumerWalletId"
   
    # @!attribute wallet_name
    # @return [String] the wallet name.
    xml_accessor :wallet_name, :from =>"walletName"
   
    # @!attribute pairing_id
    # @return [String] the new pairing token id.
    xml_accessor :pairing_id, :from =>"pairingId"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :cards => :cards ,
        :shipping_addresses => :shippingAddresses ,
        :contact_info => :contactInfo ,
        :pre_checkout_transaction_id => :preCheckoutTransactionId ,
        :consumer_wallet_id => :consumerWalletId ,
        :wallet_name => :walletName ,
        :pairing_id => :pairingId 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:cards)
        self.cards = attributes[:cards]
      end
      
      if attributes.has_key?(:shipping_addresses)
        self.shipping_addresses = attributes[:shipping_addresses]
      end
      
      if attributes.has_key?(:contact_info)
        self.contact_info = attributes[:contact_info]
      end
      
      if attributes.has_key?(:pre_checkout_transaction_id)
        self.pre_checkout_transaction_id = attributes[:pre_checkout_transaction_id]
      end
      
      if attributes.has_key?(:consumer_wallet_id)
        self.consumer_wallet_id = attributes[:consumer_wallet_id]
      end
      
      if attributes.has_key?(:wallet_name)
        self.wallet_name = attributes[:wallet_name]
      end
      
      if attributes.has_key?(:pairing_id)
        self.pairing_id = attributes[:pairing_id]
      end
      
    end
    
 
    # Overriding setter method cards
    # @param value 
  	def cards=(value)
      if (value.is_a?(Array))
        @cards = value
      else
        @cards = [] if !@cards
        @cards.push value
      end
    end
    # Overriding setter method shipping_addresses
    # @param value 
  	def shipping_addresses=(value)
      if (value.is_a?(Array))
        @shipping_addresses = value
      else
        @shipping_addresses = [] if !@shipping_addresses
        @shipping_addresses.push value
      end
    end
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          cards == o.cards &&
          shipping_addresses == o.shipping_addresses &&
          contact_info == o.contact_info &&
          pre_checkout_transaction_id == o.pre_checkout_transaction_id &&
          consumer_wallet_id == o.consumer_wallet_id &&
          wallet_name == o.wallet_name &&
          pairing_id == o.pairing_id
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [cards, shipping_addresses, contact_info, pre_checkout_transaction_id, consumer_wallet_id, wallet_name, pairing_id].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantCheckout.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "PreCheckoutData" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :cards => 'Array<PreCheckoutCard>',
        :shipping_addresses => 'Array<ShippingAddress>',
        :contact_info => 'ContactInfo',
        :pre_checkout_transaction_id => 'String',
        :consumer_wallet_id => 'String',
        :wallet_name => 'String',
        :pairing_id => 'String'
        
      }
    end
  end


end
